// -------------------------------------------------------------------------------------
//
// FishFace40.CPP                                                 Ulrich Mller 28.04.06
//                                                                PCS1700       VC++ 6.0
// CPP-Files fr die Klassen CFishFace ... Basis umFish40.DLL v4.1.59.1 (mit RF)
//
// -------------------------------------------------------------------------------------

#include <windows.h>
#include "FishFace40.h"

// ---------- CFishFaceException Constructor -------------------------------

CFishFaceException::CFishFaceException(int nr, char* text) {
	aNr   = nr;
	aText = text;
}
// ---------- CFishFace Constructor ----------------------------------------

CFishFace::CFishFace() {}
CFishFace::~CFishFace() {}

// ---------- CFishFace public Methoden ------------------------------------

DeviceData CFishFace::ActDevice() {
	if(!aOpen) throw CFishFaceException(47, "KeinOpen.ActDevice");
	DeviceData dd;
	dd.Type     = rbGetActDeviceType(iHandle);
	dd.SerialNr = rbGetActDeviceSerialNr(iHandle);
	dd.Firmware = rbGetActDeviceFirmware(iHandle);
	dd.Name     = rbGetActDeviceName(iHandle);
	return dd;
}

void CFishFace::ClearMotors() {
	if(!aOpen) throw CFishFaceException(2, "KeinOpen.ClearMotors");
	if(rbSetMotors(iHandle, 0) == rbError)
		throw CFishFaceException(3, "InterfaceProblem.ClearMotors");
}

void CFishFace::ClearMessagesIn() {
	if(!aOpen) throw CFishFaceException(42, "KeinOpen.ClearMessagesIn");
	if(rbClearMessagesIn(iHandle) == rbError)
		throw (CFishFaceException(41, "Messages.ClearMessagesIn"));
}

void CFishFace::ClearMessagesOut() {
	if(!aOpen) throw CFishFaceException(44, "KeinOpen.ClearMessagesOut");
	if(rbClearMessagesOut(iHandle) == rbError)
		throw (CFishFaceException(43, "Messages.ClearMessagesOut"));
}

void CFishFace::CloseInterface() {
	rbCloseInterface(iHandle);
	aPortNr = -1;
	aOpen   = false;
}

bool CFishFace::Finish() {
	return Finish(0);
}

bool CFishFace::Finish(int InputNr) {
	if(!aOpen) throw CFishFaceException(4, "KeinOpen.Finish");
	if(aNotHalt || GetAsyncKeyState(VK_ESCAPE) != 0) return true;
	if(rbGetInputs(iHandle) == rbError) throw CFishFaceException(5, "InterfaceProblem.Finish");
	if((InputNr > 0) && GetInput(InputNr)) return true;
	return false;
}

bool CFishFace::Finish(int IRCode, int IRKey) {
	if(!aOpen) throw CFishFaceException(45, "KeinOpen.Finish");
	if(aNotHalt || GetAsyncKeyState(VK_ESCAPE) != 0) return true;
	if(rbGetInputs(iHandle) == rbError) throw CFishFaceException(46, "InterfaceProblem.Finish");
	if(GetIRKey(IRCode, IRKey)) return true;
	return false;
}

int CFishFace::GetAnalog(int AnalogNr) {
	if(!aOpen) throw CFishFaceException(5, "KeinOpen.GetAnalog");
	int res = rbGetAnalog(iHandle, AnalogNr);
	if(res == rbError) throw CFishFaceException(6, "InterfaceProblem.GetAnalog");
	return res;
}

bool CFishFace::GetInput(int InputNr) {
	if(!aOpen) throw CFishFaceException(9, "KeinOpen.GetInput");
	if(rbGetInputs(iHandle) == rbError)
		throw CFishFaceException(10, "InterfaceProblem.GetInput");
	return (rbGetInput(iHandle, InputNr) == 0) ? false : true;
}

int CFishFace::GetInputs() {
	if(!aOpen) throw CFishFaceException(11, "KeinOpen.GetInputs");
	int res = rbGetInputs(iHandle);
	if(res == rbError) throw CFishFaceException(12, "InterfaceProblem.GetInputs");
	return res;
}

bool CFishFace::GetIRKey(int IRCode, int IRKey) {
	if(!aOpen) throw CFishFaceException(48, "KeinOpen.GetIRKey");
	int res = rbGetIRKey(iHandle, IRCode, IRKey);
	if(res == rbError) throw CFishFaceException(49, "InterfaceProblem.GetIRKey");
	return (res == 1) ? true : false;
}

MessageData CFishFace::GetRFMessage() {
	if(!aOpen) throw CFishFaceException(50, "KeinOpen.GetRFMessage");
	MessageData mD;
	if(rbGetMessage(iHandle, &mD) == rbError)
		throw CFishFaceException(51, "MessageError.GetRFMessage");
	return mD;
}

int CFishFace::GetVoltage(int VoltNr) {
	if(!aOpen) throw CFishFaceException(52, "KeinOpen.GetVoltage");
	int res = rbGetVoltage(iHandle, VoltNr);
	if(res == rbError) CFishFaceException(53, "InterfaceProblem.GetVoltage");
	return res;
}

bool CFishFace::IsRFMessage() {
	if(!aOpen) throw CFishFaceException(54, "KeinOpen.IsRFMessage");
	return (rbIsMessage(iHandle) == 1) ? true : false;
}

void CFishFace::OpenInterface(int IFSpez) {
	iHandle = rbOpenInterfaceRF(IFSpez);
	if(iHandle == rbError)
		throw CFishFaceException(62, "InterfaceProblem.OpenInterface");
	rbSetMotors(iHandle, 0);
	aPortNr       = 0;
	aNotHalt      = false;
	aOpen         = true;
	aPollInterval = 11;
}
void CFishFace::OpenInterface(int IFTyp, int IFSpez, int AnalogZyklen) {
	switch(IFTyp) {
		case ftROBO_first_USB:
		case ftROBO_IF_USB:
		case ftROBO_IF_Over_RF:
		case ftROBO_IO_Extension:
		case ftROBO_RF_Datalink:
			aAnalogScan = 0;
			iHandle = rbOpenInterfaceUSB(IFTyp, IFSpez);
			break;
		case ftIntelligent_IF:
		case ftIntelligent_IF_Slave:
		case ftROBO_IF_IIM:
		case ftROBO_IF_COM:
			aAnalogScan = AnalogZyklen;
			iHandle = rbOpenInterfaceCOM(IFTyp, IFSpez, aAnalogScan);
			break;
		default:
			return;
	}
	if(iHandle == rbError)
		throw CFishFaceException(13, "InterfaceProblem.OpenInterface");
	rbSetMotors(iHandle, 0);
	aPortNr       = IFTyp;
	aNotHalt      = false;
	aOpen         = true;
	aPollInterval = 11;
}
void CFishFace::OpenInterface(int IFTyp, int IFSpez) {
	OpenInterface(IFTyp, IFSpez, 5);
}

void CFishFace::Pause(unsigned int mSek){
	if(!aOpen) throw CFishFaceException(14, "KeinOpen.Pause");
	DWORD sZeit = GetTickCount();
	do {
		if(GetInputs() == rbError)
			throw CFishFaceException(15, "InterfaceProblem.Pause");
		if(aNotHalt || GetAsyncKeyState(VK_ESCAPE) != 0) return;
		Sleep(aPollInterval);
	} while((GetTickCount() - sZeit) < mSek);
}

void CFishFace::SendRFMessage(MessageData outN, int Spez) {
	if(!aOpen) throw CFishFaceException(56, "KeinOpen.SendRFMessage");
	if(rbSendMessageEx(iHandle, &outN, Spez) == rbError)
		throw CFishFaceException(57, "MessageError.SendRFMessage");
}
void CFishFace::SendRFMessage(MessageData outN) {
	SendRFMessage(outN, 0);
}

void CFishFace::SetLamp(int LampNr, int OnOff) {
	if(!aOpen) throw CFishFaceException(16, "KeinOpen.SetLamp");
	if(rbSetLamp(iHandle, LampNr, OnOff) == rbError)
		throw CFishFaceException(17, "InterfaceProblem.SetLamp");
}

void CFishFace::SetMotor(int MotorNr, int Direction) {
	if(!aOpen) throw CFishFaceException(17, "KeinOpen.SetMotor");
	if(rbSetMotor(iHandle, MotorNr, Direction) == rbError)
		throw CFishFaceException(18, "Interface.Problem");
}

void CFishFace::SetMotor(int MotorNr, int Direction, int Speed) {
	if(!aOpen) throw CFishFaceException(19, "KeinOpen.SetMotor");
	if(rbSetMotorEx(iHandle, MotorNr, Direction, Speed) == rbError)
		throw CFishFaceException(20, "InterfaceProblem.SetMotor");
}

void CFishFace::SetMotor(int MotorNr, int Direction, int Speed, int Counter) {
	if(!aOpen) throw CFishFaceException(21, "KeinOpen.SetMotor");
	if(rbRobMotor(iHandle, MotorNr, Direction, Speed, Counter) == rbError)
		throw CFishFaceException(22, "InterfaceProblem.SetMotor");
}

void CFishFace::SetMotors(int MotorStatus) {
	if(!aOpen) throw CFishFaceException(23, "KeinOpen.SetMotors");
	if(rbSetMotors(iHandle, MotorStatus) == rbError) 
		throw CFishFaceException(24, "InterfaceProblem.SetMotors");
}

void CFishFace::SetMotors(int MotorStatus, int SpeedStatus, int SpeedStatus16) {
	if(!aOpen) throw CFishFaceException(25, "KeinOpen.SetMotors");
	if(rbSetMotorsEx(iHandle, MotorStatus, SpeedStatus, SpeedStatus16) == rbError) 
		throw CFishFaceException(26, "InterfaceProblem.SetMotors");
}

void CFishFace::SetMotors(int MotorStatus, int SpeedStatus, int SpeedStatus16, int ModeStatus) {
	if(!aOpen) throw CFishFaceException(27, "KeinOpen.SetMotors");
	if(rbRobMotors(iHandle, MotorStatus, SpeedStatus, SpeedStatus16, ModeStatus) == rbError) 
		throw CFishFaceException(28, "InterfaceProblem.SetMotors");
}

void CFishFace::WaitForChange(int InputNr, int NrOfChanges, int TermInputNr) {
	if(!aOpen) throw CFishFaceException(29, "KeinOpen.WaitForChange");
	rbSetCounter(iHandle, InputNr, 0);
	while(rbGetCounter(iHandle, InputNr) < NrOfChanges) {
		if(aNotHalt || GetAsyncKeyState(VK_ESCAPE) != 0) return;
		if(TermInputNr > 0) if(GetInput(TermInputNr)) return;
		if(GetInputs() == rbError)
			throw CFishFaceException(30, "InterfaceProblem.WaitForChange");
	}
}

void CFishFace::WaitForHigh(int InputNr) {
	WaitForInput(InputNr, false);
	WaitForInput(InputNr, true);
}

void CFishFace::WaitForInput(int InputNr, bool OnOff) {
	if(!aOpen) throw CFishFaceException(31, "KeinOpen.WaitForInput");
	do {
		if(aNotHalt || GetAsyncKeyState(VK_ESCAPE) != 0) return;
		if(GetInputs() == rbError)
			throw CFishFaceException(32, "InterfaceProblem.WaitForInput");
		Sleep(aPollInterval);
	} while(!GetInput(InputNr) == OnOff);
}

void CFishFace::WaitForInput(int IRCode, int IRKeyNr, bool OnOff) {
	if(!aOpen) throw CFishFaceException(58, "KeinOpen.WaitForInput");
	do {
		if(aNotHalt || GetAsyncKeyState(VK_ESCAPE) != 0) return;
		if(GetInputs() == rbError)
			throw CFishFaceException(59, "InterfaceProblem.WaitForInput");
		Sleep(aPollInterval);
	} while(!GetIRKey(IRCode, IRKeyNr) == OnOff);
}

void CFishFace::WaitForLow(int InputNr) {
	WaitForInput(InputNr, true);
	WaitForInput(InputNr, false);
}

int  CFishFace::WaitForRFMessage(int mSek, MessageData mData) {
	unsigned int t;
	mData.HwId  = 0;
	mData.SubId = 0;
	mData.MsgId = 0;
	mData.Msg   = 0;
	if(!aOpen) throw CFishFaceException(60, "KeinOpen.WaitForRFMessage");
	t = GetTickCount() + mSek;
	do {
		if(rbIsMessage(iHandle) != 0) {
			if(rbGetMessage(iHandle, &mData) == rbError)
				throw CFishFaceException(61, "NachrichtenFehler.WaitForRFMessage");
			return waiEnd;
		}
		if(aNotHalt) return waiNotHalt;
		if(GetAsyncKeyState(VK_ESCAPE) != 0) return waiESC;
		Sleep(aPollInterval);
	} while(!mSek || (t > GetTickCount()));
	return waiTime;
}

int CFishFace::WaitForMotors(unsigned int mSek, int M1, int M2, int M3, int M4, 
														 int M5, int M6, int M7, int M8) {
	const int mM3[] = {0, 0x3, 0xC, 0x30, 0xC0, 0x300, 0xC00, 0x3000, 0xC000};
	int c;
	unsigned int t;
	if(!aOpen) throw CFishFaceException(33, "KeinOpen.WaitForMotors");
	t = GetTickCount() + mSek;
	do {
		c = 0;
		if(M1) c += rbGetMotors(iHandle) & mM3[M1];
		if(M2) c += rbGetMotors(iHandle) & mM3[M2];
		if(M3) c += rbGetMotors(iHandle) & mM3[M3];
		if(M4) c += rbGetMotors(iHandle) & mM3[M4];
		if(M5) c += rbGetMotors(iHandle) & mM3[M5];
		if(M6) c += rbGetMotors(iHandle) & mM3[M6];
		if(M7) c += rbGetMotors(iHandle) & mM3[M7];
		if(M8) c += rbGetMotors(iHandle) & mM3[M8];

		if(!c) return waiEnd;

		// --- Abschalten der Motoren der Parameterleiste -----
		if(aNotHalt || GetAsyncKeyState(VK_ESCAPE) != 0) {
			if(M1) rbSetMotor(iHandle, M1, dirOff);
			if(M2) rbSetMotor(iHandle, M2, dirOff);
			if(M3) rbSetMotor(iHandle, M3, dirOff);
			if(M4) rbSetMotor(iHandle, M4, dirOff);
			if(M5) rbSetMotor(iHandle, M5, dirOff);
			if(M6) rbSetMotor(iHandle, M6, dirOff);
			if(M7) rbSetMotor(iHandle, M7, dirOff);
			if(M8) rbSetMotor(iHandle, M8, dirOff);
		}
		if(GetInputs() == rbError)
			throw CFishFaceException(34, "InterfaceProblem.WaitForMotors");
		if(aNotHalt) return waiNotHalt;
		if(GetAsyncKeyState(VK_ESCAPE) != 0)  return waiESC;
		Sleep(aPollInterval);
	} while(!mSek || (t > GetTickCount()));
	return waiTime;
}

int CFishFace::WaitForPositionDown(int InputNr, int Counter, int Position, int TermInputNr) {
	if(!aOpen) throw CFishFaceException(35, "KeinOpen.WaitForPositionDown");
	WaitForChange(InputNr, Counter - Position, TermInputNr);
	return Counter - rbGetCounter(iHandle, InputNr);
}

int CFishFace::WaitForPositionUp(int InputNr, int Counter, int Position, int TermInputNr) {
	if(!aOpen) throw CFishFaceException(36, "KeinOpen.WaitForPositionUp");
	WaitForChange(InputNr, Position - Counter, TermInputNr);
	return rbGetCounter(iHandle, InputNr) + Counter;
}

void CFishFace::WaitForTime(unsigned int mSek) {
	if(!aOpen) throw CFishFaceException(37, "KeinOpen.WaitForTime");
	Pause(mSek);
}

